#include "runtime/mem_arena.h"
#include "runtime/resources.h"
#include "runtime/threading.h"

#include "gfx/gfx.h"
#include "gfx/renderer_api.h"

#include "asset_compiler/asset_compiler.h"

void RegisterCVars(void) {
    rtRegisterAssetCompilerCVars();
}

static rt_framegraph *_framegraph;

static void PassPrepare(rt_render_pass_id pass,
                        const rt_render_target_write *writes,
                        uint32_t write_count,
                        const rt_render_target_read *reads,
                        uint32_t read_count) {
    //rtLog("GAME", "Prepare pass %x", pass);
}

static void PassExecute(rt_render_pass_id pass,
                        const rt_render_target_write *writes,
                        uint32_t write_count,
                        const rt_render_target_read *reads,
                        uint32_t read_count) {
    //rtLog("GAME", "Execute pass %x", pass);
}

static void PassFinalize(rt_render_pass_id pass,
                         const rt_render_target_write *writes,
                         uint32_t write_count,
                         const rt_render_target_read *reads,
                         uint32_t read_count) {
    //rtLog("GAME", "Finalize pass %x", pass);
}

/* Called after the runtime has finished its initialization and before entering the main-loop*/
void Init(void) {
    rtLog("GAME", "Init");
    rtInitAssetCompiler();

    rtWaitForAssetProcessing();

    rt_temp_arena temp = rtGetTemporaryArena(NULL, 0);

    rt_resource_id resid = rtGetResourceID("assets/test.framegraph");
    size_t size          = rtGetResourceSize(resid);
    rt_resource *res     = rtArenaPush(temp.arena, size);
    rtGetResource(resid, res);

    _framegraph = rtCreateFramegraph(res->data);

    rt_render_pass_bind_fns bind = {.Execute  = PassExecute,
                                    .Prepare  = PassPrepare,
                                    .Finalize = PassFinalize};
    rtBindRenderPass(_framegraph, rtCalculateRenderPassID("pass0", sizeof("pass0") - 1), &bind);
    rtBindRenderPass(_framegraph, rtCalculateRenderPassID("pass1", sizeof("pass1") - 1), &bind);
}

/* Called after exiting the main-loop and before the runtime starts its shutdown */
void Shutdown(void) {
    rtLog("GAME", "Shutdown");
    rtShutdownAssetCompiler();
    rtDestroyFramegraph(_framegraph);
}

void Update(unsigned int frame_id) {
    RT_UNUSED(frame_id);
}

void Render(unsigned int frame_id) {
    rtExecuteFramegraph(_framegraph, frame_id);
}